%Dino Sejdinovic, 2013
%D. Sejdinovic, A. Gretton and W. Bergsma.  A KERNEL TEST FOR THREE-VARIABLE INTERACTIONS, 2013.
%runs three-variable interaction tests on synthetic data

%input format
%---which_data: character indicating the choice of a synthetic data generator
%---------------currently available: 'A', 'B'
%---dim:        dimensionality of random vectors
%---namebase [optional]: prefix for filename where the results will be saved
%------------------------if not specified, the results are not saved (they are still returned) 

%output format
%--------param: the structure of all parameters used in the test
%--------pvalues: the structure of all pvalues obtained in the tests,
%--------rejectionrate: the rates at which null is rejected in the tests
%-----------------------(averages over number of trials)
%--------pvalues and rejectionrate may contain some or all of substructures
%-----------------------TwoVar, Lancaster, Total, CI
%--------access individual pvalue vectors with, e.g., pvalues.TwoVar.XY_Z

%CI code is from [Zhang et al, 2011] ---- code available at:
%--------http://people.tuebingen.mpg.de/kzhang/KCI-test.zip
%we use the function indtest_new which can be found in '../KCI_test/'

function [param,pvalues,rejectionrate]=synthetic_experiment(which_data,dim,namebase)
       

if nargin==2
    namebase=[];
    save_results=false;
else
    save_results=true;
end


param.num_shuffles=2000;        %number of shuffles in the permutation test
param.use_median=true;          %using median heuristic for kernel bandwidth
param.alpha=0.05;               %confidence level
param.num_trials=2;             %number of trials to be run
param.noise_std=0;              %standard deviation of noise added
param.sample_size=500;          %sample size
param.which_data=which_data;    %choose between synthetic data generators
param.dim=dim;                  %dimensionality of random vectors

%specify which tests should be run below:
param.run_lancaster=true;
param.run_twovar=true;
param.run_total=true;
param.run_ci=true;
%if it is apriori known that X\indep Y, test for CI X\indep Y | Z
%becomes test for the V-structure X->Z<-Y, 
%V-structure in this case can also be tested with "Factorization" tests
if param.run_ci
    addpath(genpath('../KCI_test'));
end



for jj=1:param.num_trials
    
    fprintf(strcat('Trial', num2str(jj),'\n'));
    
    if      strcmp(which_data,'A')
        [xxn,yyn,zzn] = generate_dataA(param.sample_size,param.dim);
    elseif  strcmp(which_data, 'B')
        [xxn,yyn,zzn] = generate_dataB(param.sample_size,param.dim);
    end

    
    %add noise to check robustness if required
    if param.noise_std>0
        xxn=xxn+param.noise_std*randn(size(xxn));
        yyn=yyn+param.noise_std*randn(size(yyn));
        zzn=zzn+param.noise_std*randn(size(zzn));
    end
    
    if param.run_twovar
    results(jj).TwoVar   =  all_two_var_tests(xxn,yyn,zzn,param);
    end
    
    if param.run_lancaster
    results(jj).Lancaster = lancaster_tests(xxn,yyn,zzn,param);
    end
    
    if param.run_total
    results(jj).Total = total_independence_test(xxn,yyn,zzn,param);
    end
    
    if param.run_ci
    %calling the UAI2011 CI test for "X independent of Y given Z"
    results(jj).CI.XYgivenZ.pvalue = indtest_new(xxn,yyn,zzn,[]);
    end
    
end

[pvalues,rejectionrate] = reshape_results(results,param);

if save_results
filename=[namebase 'SynthData' param.which_data '_dim' num2str(param.dim)]; 
save(['results/' filename],'param','pvalues','rejectionrate');
end

